/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.token;

import de.ozgcloud.token.saml.SamlTokenService;
import io.grpc.stub.StreamObserver;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.server.service.GrpcService;

@Log4j2
@GrpcService
@RequiredArgsConstructor
public class TokenCheckGrpcService extends TokenCheckServiceGrpc.TokenCheckServiceImplBase {

	private final SamlTokenService samlTokenService;
	private final TokenValidationResultMapper tokenCheckMapper;

	@Override
	public void checkToken(GrpcCheckTokenRequest request, StreamObserver<GrpcCheckTokenResponse> responseStreamObserver) {
		var result = samlTokenService.validate(request.getToken());
		responseStreamObserver.onNext(buildCheckResponse(result));
		responseStreamObserver.onCompleted();
	}

	GrpcCheckTokenResponse buildCheckResponse(TokenValidationResult result) {
		return result.isValid() ? buildValidCheckTokenResponse(result) : buildInvalidCheckTokenResponse(result);
	}

	GrpcCheckTokenResponse buildValidCheckTokenResponse(TokenValidationResult result) {
		return GrpcCheckTokenResponse.newBuilder()
				.setTokenValid(true)
				.setTokenAttributes(tokenCheckMapper.toTokenAttributes(result)).build();
	}

	GrpcCheckTokenResponse buildInvalidCheckTokenResponse(TokenValidationResult result) {
		return GrpcCheckTokenResponse.newBuilder()
				.setTokenValid(false)
				.setCheckErrors(tokenCheckMapper.toCheckErrors(result))
				.build();
	}
}
