/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.token;

import java.util.Map;

import org.opensaml.core.config.ConfigurationService;
import org.opensaml.core.config.InitializationException;
import org.opensaml.core.config.InitializationService;
import org.opensaml.core.xml.config.XMLObjectProviderRegistry;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import net.shibboleth.utilities.java.support.component.ComponentInitializationException;
import net.shibboleth.utilities.java.support.xml.BasicParserPool;
import net.shibboleth.utilities.java.support.xml.ParserPool;

@Configuration
public class TokenCheckerConfiguration {

	public static final String FEATURES_EXTERNAL_GENERAL_ENTITIES = "http://xml.org/sax/features/external-general-entities";
	public static final String FEATURES_EXTERNAL_PARAMETER_ENTITIES = "http://xml.org/sax/features/external-parameter-entities";
	public static final String FEATURES_DISALLOW_DOCTYPE_DECL = "http://apache.org/xml/features/disallow-doctype-decl";
	public static final String VALIDATION_SCHEMA_NORMALIZED_VALUE = "http://apache.org/xml/features/validation/schema/normalized-value";
	public static final String FEATURE_SECURE_PROCESSING = "http://javax.xml.XMLConstants/feature/secure-processing";

	@Bean
	ParserPool parserPool() throws ComponentInitializationException {
		var localParserPool = new BasicParserPool();
		localParserPool.setBuilderFeatures(createFeatureMap());
		localParserPool.initialize();

		return localParserPool;
	}

	Map<String, Boolean> createFeatureMap() {
		return Map.of(
				FEATURES_EXTERNAL_GENERAL_ENTITIES, Boolean.FALSE,
				FEATURES_EXTERNAL_PARAMETER_ENTITIES, Boolean.FALSE,
				FEATURES_DISALLOW_DOCTYPE_DECL, Boolean.TRUE,
				VALIDATION_SCHEMA_NORMALIZED_VALUE, Boolean.FALSE,
				FEATURE_SECURE_PROCESSING, Boolean.TRUE);
	}

	@Bean
	XMLObjectProviderRegistry xmlObjectProviderRegistry(ParserPool parserPool) throws InitializationException {
		var registry = new XMLObjectProviderRegistry();
		registry.setParserPool(parserPool);
		ConfigurationService.register(XMLObjectProviderRegistry.class, registry);
		InitializationService.initialize();
		return registry;
	}

}
