/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.token;

import java.util.List;
import java.util.Map;

import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import jakarta.validation.constraints.NotNull;

import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.Resource;

import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

@Setter
@Getter
@Configuration
@ConfigurationProperties(prefix = TokenValidationProperties.PREFIX)
public class TokenValidationProperties {

	static final String PREFIX = "ozgcloud.token.check";
	/**
	 * List of entities. A ConfigurationEntity contains the necessary information for verifying and decrypting saml tokens.
	 */
	@NotEmpty
	@Valid
	private List<TokenValidationProperty> entities;

	@Getter
	@Setter
	@ToString
	public static class TokenValidationProperty {

		/**
		 * The id of the Identity Provider, this is also the issuer value.
		 */
		@NotEmpty
		private String idpEntityId;

		/**
		 * The encryption key
		 */
		@NotEmpty
		private Resource key;

		/**
		 * The encryption certificate
		 */
		@NotEmpty
		private Resource certificate;

		/**
		 * The url or the actual SAML Metadata file received from the idp
		 */
		@NotEmpty
		private Resource metadata;

		/**
		 * Use the user id as Postkorbhandle. For Muk
		 */
		private boolean useIdAsPostfachId = false;

		/**
		 * The mappings define the names of the attributes in the SamlResponse that correspond to these keys
		 */
		@NotNull
		private Map<String, String> mappings;
	}
}
