/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.token.common.errorhandling;

import java.io.Serial;
import java.util.Collections;
import java.util.Map;
import java.util.UUID;

import de.ozgcloud.common.errorhandling.ExceptionUtil;
import de.ozgcloud.common.errorhandling.FunctionalErrorCode;
import de.ozgcloud.common.errorhandling.IdentifiableException;
import lombok.Getter;

public class FunctionalException extends RuntimeException implements IdentifiableException {
	@Serial
	private static final long serialVersionUID = 1L;

	private static final String LOG_MESSAGE_TEMPLATE = "FunctionalException with ErrorCode '%s'.";
	private final String exceptionId;

	private final FunctionalErrorCode errorCode;
	@Getter
	private final Map<String, String> parameters;

	public FunctionalException(FunctionalErrorCode errorCode) {
		this(errorCode, Collections.emptyMap());
	}

	public FunctionalException(FunctionalErrorCode errorCode, Map<String, String> parameters) {
		this(String.format(LOG_MESSAGE_TEMPLATE, errorCode.getErrorCode()), errorCode, parameters);
	}

	public FunctionalException(String logMessage, FunctionalErrorCode errorCode) {
		this(logMessage, errorCode, Collections.emptyMap());
	}

	public FunctionalException(String logMessage, FunctionalErrorCode errorCode, Map<String, String> parameters) {
		super(logMessage);

		this.errorCode = errorCode;
		this.parameters = Collections.unmodifiableMap(parameters);
		this.exceptionId = UUID.randomUUID().toString();
	}

	public String getErrorCode() {
		return errorCode.getErrorCode();
	}

	@Override
	public String getMessage() {
		return ExceptionUtil.formatMessageWithExceptionId(super.getMessage(), exceptionId);
	}

	@Override
	public String getExceptionId() {
		return exceptionId;
	}
}