/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.token.common.errorhandling;

import java.util.Collections;
import java.util.List;
import java.util.Objects;

import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.Getter;

@Getter
public class TokenVerificationException extends TechnicalException {

	private final List<ValidationError> validationErrors;

	public TokenVerificationException(String msg) {
		super(msg);
		this.validationErrors = Collections.singletonList(ValidationError.builder().message(msg).build());
	}

	public TokenVerificationException(String msg, List<ValidationError> validationErrors) {
		super(msg);
		this.validationErrors = Objects.isNull(validationErrors) ? Collections.emptyList() : Collections.unmodifiableList(validationErrors);
	}

	public TokenVerificationException(String msg, Throwable exception) {
		super(msg, exception);
		this.validationErrors = Collections.singletonList(ValidationError.builder().message(msg).build());
	}

	@Override
	public String getMessage() {
		return "[SAML]" + super.getMessage();
	}
}
