/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.token;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import de.ozgcloud.token.common.errorHandler.ValidationErrorTestFactory;

class TokenValidationResultMapperTest {

	private final TokenValidationResultMapper mapper = spy(Mappers.getMapper(TokenValidationResultMapper.class));

	@Nested
	class TestToTokenAttributes {

		@Test
		void shouldMapTokenAttributes() {
			var result = mapper.toTokenAttributes(TokenValidationResultTestFactory.createValid());

			assertThat(result).usingRecursiveComparison().isEqualTo(GrpcTokenAttributesTestFactory.create());
		}
	}

	@Nested
	class TestToCheckErrors {

		@BeforeEach
		void init() {
			doReturn(GrpcCheckErrorTestFactory.create()).when(mapper).toCheckError(any());
		}

		@Test
		void shouldMapCheckErrors() {
			var result = mapper.toCheckErrors(TokenValidationResultTestFactory.createInvalid());

			assertThat(result).usingRecursiveComparison().isEqualTo(GrpcCheckErrorsTestFactory.create());
		}
	}

	@Nested
	class TestToCheckError {

		@Test
		void shouldMapCheckError() {
			var result = mapper.toCheckError(ValidationErrorTestFactory.create());

			assertThat(result).usingRecursiveComparison().isEqualTo(GrpcCheckErrorTestFactory.create());
		}
	}
}